// ********************************************************************
// VNC1L/VDIP1 - Grundfunktionen 
// AVR-Version / GGC 4.3.2
// Version 1.00 / Stand: 20.10.2010
// Autor: Matthias Kahnt
// ********************************************************************
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// Hinweis: Exotische Sticks (z.B. DualCanal OZC) lassen sich vom VNC1L
// bisher nicht beschreiben. Getestete USB-Sticks:
// RUNDISK 512MB FAT16 - ok
// PLATINUM  2GB FAT32 - ok
// CNMEMORY  4GB FAT32 - ok
// TREKSTOR 16GB FAT32 - ok
// ********************************************************************
// Aktuelle VDAP Firmware muss auf den VDIP1/VNC1L installiert sein,
// getestet mit Version 3.68. Achtung: Auf neuen VDIP1 ist meist eine
// sehr alte Version drauf, also erstmal ein Firmware-Update machen.
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// des Autors erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
// ***********************************************************************
// VNC1L - USB HOST CONTROLLER - SPI-MODE (MC: Master, VNC1L: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.68 getestet
// ***********************************************************************
//
//************************************************************************
// AVR/GCC LIBS
//************************************************************************
#include "defines.h"            // Projektdefinitionen
#include <avr/io.h>             // IO-Grundfunktionen
#include <util/delay.h>         // Pausenfunktionen
#include <string.h>             // Stringfunktion
#include <stdio.h>              // Standardfunktion

#include "dip204spi_avr.h"      // LCD-Funktionen
#include "ULOG_AVR_Projekt.h"   // Projekt-Funktionen

// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// ***********************************************************************
void vnc_init(void) {
  CS_VNC_L;         // SPI-Interface inaktiv
  SCLK_L;           // Ruhezustand der SPI-Leitungen herstellen
  SDI_H;
  RE_VNC_L;         // Hardware-Reset VNC1L
  _delay_ms(100);   // Pause
  RE_VNC_H;         // VNC1L starten
  _delay_ms(500);   // Pause fr Anlauf
}

// ***********************************************************************
// VNC1L - 13-Bit-SPI Senden eines Zeichens
// SPI Mode
// bergabe: wr_byte : Auszugebendes Zeichen
// Rckgabe: status  : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_spi(unsigned char wr_byte) {
unsigned char stelle, status, cnt;
  cnt = TIMEOUT_SPI;            // Anzahl der Schreib-Versuche bis Timeout
  do {
    CS_VNC_H;                   // SPI-Interface aktiv
    SDI_H;                      // Startbit (immer 1)
    SCLK_H;  SCLK_L;            // L/H Flanke
    SDI_L;                      // RW-Bit (0=Write)
    SCLK_H;  SCLK_L;            // L/H Flanke
    SDI_L;                      // ADDR (0=Daten)
    SCLK_H;  SCLK_L;            // L/H Flanke
    // 8 Datenbits senden (MSB zuerst)
    for ( stelle = 0x80; stelle; stelle >>= 1 ) {
      if (wr_byte & stelle) SDI_H; else SDI_L;
      SCLK_H;  SCLK_L;          // L/H Flanke
    }
    // Statusbit einlesen
    status = SDO_VNC;           // Statusbit (0=Accept, 1=Reject)
    SCLK_H;  SCLK_L;            // L/H Flanke
    // End Sequenz
    CS_VNC_L;                   // SPI-Interface inaktiv
    SCLK_H;  SCLK_L;            // L/H Flanke
    cnt--;                      // Zhler der Schreibversuche
    if (status) _delay_us(150); // Pause, wenn Byte nicht besttigt wurde
  } while (status && cnt);

  #ifdef LCD_MONITORING
    lcd_monitor_byte(wr_byte); // gesendetes Zeichen auf LCD anzeigen
  #endif

  if (awd.stick_speed) _delay_us(100);  // USB-Speed/Timing bremsen bei Bedarf
  return status;
} 

// ***********************************************************************
// VNC1L - 13-Bit-SPI Empfangen eines Zeichens
// SPI Mode
// bergabe: rd_fkt : Funktion (0=Datenbyte, 1=Statusbyte)
// Datenbyte wird im Buffer vnc-Stukturvariable gespeichert
// Rckgabe: status : 0=Neues Byte im Buffer (ok) 1=kein neues Byte vom VNC1L vorhanden
// ***********************************************************************
unsigned char vnc_rd_spi(unsigned char rd_fkt) {
unsigned char stelle, daten, status;
unsigned int cnt;
  cnt = TIMEOUT_SPI;                //  Anzahl der Lese-Versuche bis Timeout
  do {
    daten  = 0;
    CS_VNC_H;                       // SPI-Interface aktiv
    SDI_H;                          // Startbit (immer 1)
    SCLK_H;  SCLK_L;                // L/H Flanke
    SDI_H;                          // RW-Bit (1=Read)
    SCLK_H;  SCLK_L;                // L/H Flanke
    if (rd_fkt) SDI_H; else SDI_L;  // ADDR (0=Daten, 1=SPI-Statusinformation) 
    SCLK_H;  SCLK_L;                // L/H Flanke
    // 8 Datenbits empfangen (MSB zuerst)
    for ( stelle = 0x80; stelle; stelle >>= 1 ) {
      if (SDO_VNC) daten |= stelle; // empfangenes Datenbit speichern
      SCLK_H;  SCLK_L;              // L/H Flanke
    }
    // Statusbit einlesen
    status = SDO_VNC;               // Statusbit (0=New Data, 1=Old Data)
    SCLK_H;  SCLK_L;                // L/H Flanke
    // End Sequenz
    CS_VNC_L;                       // SPI-Interface inaktiv
    SCLK_H;  SCLK_L;                // L/H Flanke
    // SDO    = 1;                  // nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
    if (!status) {
      // empfangenes Zeichen ist ein neues Zeichen
      // Empfangsbuffer um eine Stelle verschieben
      for ( stelle = VNC_BUFFER_SIZE - 1; stelle; stelle-- ) vnc.rec_buffer[stelle] = vnc.rec_buffer[stelle - 1];
      vnc.rec_buffer[0] = daten;    // neues Zeichen an 1. Stelle speichern

      #ifdef LCD_MONITORING
        if (!rd_fkt) {
          lcd_monitor_byte(vnc.rec_buffer[0]); // empfangenes Zeichen auf LCD anzeigen
        }
      #endif

    }
    cnt--;                          // Zhler der Leseversuche
    if (status) _delay_us(150);     // Pause, wenn keine neues Byte vorliegt
  } while (status && cnt);
  if (awd.stick_speed) _delay_us(100);  // USB-Speed/Timing bremsen bei Bedarf
  return status;
} 

// ***********************************************************************
// VNC1L - Senden eines Zeichens
// bergabe: wr_byte : Auszugebendes Zeichen
// Rckgabe: status  : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_byte(unsigned char wr_byte) {
unsigned char status;
unsigned int cnt;
  cnt = TIMEOUT_READ_WRITE;             // Anzahl der Sende-Versuche bis Timeout
  do {
    // VNC1L Status-Byte lesen
    if (vnc_rd_spi(1)) return 1;        // Fehler, Byte wurde nicht angenommen
    // Statusbyte liegt jetzt im Buffer
    status = vnc.rec_buffer[0] & 0x01;  // Test auf VNC1L RECEIVE-BUFFER voll (RXF# Bit0=1)
    cnt--;                              // Zhler der Schreibversuche
    if (status) _delay_us(150);         // Pause, wenn RECEIVE-BUFFER voll (RXF#)
  } while (status && cnt);
  if (status || (!cnt)) return 1;       // Fehler, Byte wurde nicht angenommen
  // VNC1L ist jetzt empfangsbereit
  if (vnc_wr_spi(wr_byte)) return 1;    // Fehler Byte wurde nicht angenommen
  if (awd.stick_speed) _delay_ms(1);    // USB-Speed/Timing bremsen bei Bedarf
  return 0;                             // Byte wurde gesendet
}

// ***********************************************************************
// VNC1L - Empfangen eines Zeichens
// Rckgabe: status : 0=Neues Byte im Buffer (ok) 1=kein neues Byte vom VNC1L vorhanden
// ***********************************************************************
unsigned char vnc_rd_byte(void) {
unsigned char status;
unsigned int cnt;
  cnt = TIMEOUT_READ_WRITE;             // Anzahl der Empfangs-Versuche bis Timeout
  do {
    // VNC1L Status-Byte lesen
    status = vnc_rd_spi(0);             // Wenn Status=1 liegt kein neues Byte vor
    // Prfung auf TXE# im Status-Byte brachte keinen Vorteil
    cnt--;                              // Zhler der Leseversuche
    if (status) _delay_us(150);         // Pause, wenn kein neue Byte vorliegt
  } while (status && cnt);
  if (status || (!cnt)) return 1;       // Fehler, kein Byte empfangen
  if (awd.stick_speed) _delay_ms(1);    // USB-Speed/Timing bremsen bei Bedarf
  return 0;                             // Byte wurde empfangen
}

// ***********************************************************************
// VNC1L - Senden eines Kommandos
// bergabe: wr_cmd : Kommando (Adresse)
// Rckgabe: status : 0=ok 1=Kommando nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_cmd(char *wr_cmd) {
unsigned char vnc_i;
  for(vnc_i = strlen(wr_cmd); vnc_i; vnc_i--) {
    if (vnc_wr_byte(*wr_cmd)) return 1; // Fehler, berlauf, Kmdo. wurde nicht angenommen
    wr_cmd++;
    if (awd.stick_speed) _delay_ms(1);  // USB-Speed/Timing bremsen bei Bedarf
  }
  if (vnc_wr_byte(0x0D)) return 1;      // Fehler berlauf, Kmdo. wurde nicht angenommen
  if (awd.stick_speed) _delay_ms(10);   // USB-Speed/Timing bremsen bei Bedarf
  return 0;
}

// ***********************************************************************
// VNC1L - String empfangen
// Letzte 12 Stellen sind signifikant
// bergabe: wr_string : String (Adresse)
// Rckgabe: status    : 0=ok 1=String nicht vom VNC1L empfangen
// ***********************************************************************
unsigned char vnc_wait_for(char *wr_string) {
unsigned char laenge;       // Stringlnge/Offset
unsigned char treff_index;  // Anzahl der bereinstimmungen
unsigned char buff_index;   // Buffer-Index
unsigned char i;
unsigned char status;
unsigned int rd_answer_versuche;
  rd_answer_versuche = TIMEOUT_KOMMUNIKATION; // TIMEOUT bis Fehler/Abbruch ggf.
  status       = 0;
  treff_index  = 0;
  buff_index   = 0;
  laenge = strlen(wr_string);
  if (laenge > 12) laenge = 12; // Lnge begrenzen
  while(rd_answer_versuche && (laenge != treff_index)) {
    status = vnc_rd_byte();
    if (!status) {
      // neues Zeichen liegt jetzt vor
      buff_index   = 0;
      treff_index  = 0;
      for(i = 0; i < laenge; i++) {
        if (vnc.rec_buffer[laenge - i] == *(wr_string + i)) treff_index++; // bereinstimmung
        buff_index++;
      }
      rd_answer_versuche = TIMEOUT_KOMMUNIKATION;   // Timeout rcksetzen
    } else rd_answer_versuche--;
  }
  if (status || (treff_index != laenge)) return 1;  // Fehler, String wurde nicht empfangen
  return 0;                                         // String wurde empfangen ok
}

// ***********************************************************************
// VNC1L - Auf Antwortstring warten und analysieren
// Die lezten 4 Zeichen der Antwort sind signifikant, ggf. erweitern!)
// bergabe: 1 = Prfung auf Prompt und Fehlermeldungen
//           2 = Prfung nur auf Prompt
// Rckgabe: 1 = Prompt empfangen
//           2 = Command Failed empfangen
//           3 = Bad Command
//           4 = Disk Full
//           5 = Filename Invalid
//           6 = Read Only
//           7 = File Open
//           8 = Dir Not Empty
//           9 = No Disk
//          99 = Timeout
// ***********************************************************************
unsigned char vnc_rd_answer(unsigned char buffer_check) {
unsigned char antw;
unsigned char status;
unsigned int rd_answer_versuche;
  rd_answer_versuche = TIMEOUT_KOMMUNIKATION; // TIMEOUT bis Fehler/Abbruch ggf. ANPASSEN!
  antw = 0;
  for (status = VNC_BUFFER_SIZE - 1; status; status-- ) vnc.rec_buffer[status] = 0; // Buffer lschen, status-Variable genutzt
  while(rd_answer_versuche) {
    status = vnc_rd_byte();
    if (!status) {
      // neues Zeichen liegt jetzt vor
      switch (buffer_check) {
          case 1: // Command Failed
                  if ((vnc.rec_buffer[3] == 'i') && (vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'd')) antw = 2;
                  // Bad Command
                  if ((vnc.rec_buffer[3] == 'm') && (vnc.rec_buffer[2] == 'a') && (vnc.rec_buffer[1] == 'n') && (vnc.rec_buffer[0] == 'd')) antw = 3;
                  // Disk Full
                  if ((vnc.rec_buffer[3] == 'F') && (vnc.rec_buffer[2] == 'u') && (vnc.rec_buffer[1] == 'l') && (vnc.rec_buffer[0] == 'l')) antw = 4;
                  // Invalid Filename Invalid
                  if ((vnc.rec_buffer[3] == 'a') && (vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'i') && (vnc.rec_buffer[0] == 'd')) antw = 5;
                  // Read Only
                  if ((vnc.rec_buffer[3] == 'O') && (vnc.rec_buffer[2] == 'n') && (vnc.rec_buffer[1] == 'l') && (vnc.rec_buffer[0] == 'y')) antw = 6;
                  // File Open
                  if ((vnc.rec_buffer[3] == 'O') && (vnc.rec_buffer[2] == 'p') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'n')) antw = 7;
                  // Dir Not Empty
                  if ((vnc.rec_buffer[3] == 'm') && (vnc.rec_buffer[2] == 'p') && (vnc.rec_buffer[1] == 't') && (vnc.rec_buffer[0] == 'y')) antw = 8;
                  // No Disk
                  if ((vnc.rec_buffer[3] == 'D') && (vnc.rec_buffer[2] == 'i') && (vnc.rec_buffer[1] == 's') && (vnc.rec_buffer[0] == 'k')) antw = 9;
                  // hier knnte auf weitere Fehlermeldungen gecheckt werden...
                  // kein break; hier!!!
          case 2: // nur Prompt Check
                  if ((vnc.rec_buffer[2] == 0x5C) && (vnc.rec_buffer[1] == '>') && (vnc.rec_buffer[0] == 0x0D)) {
                    // Test auf \> und CR war ok
                    antw = 1;
                    //vnc.rec_buffer[0] = 0; // Buffer verwerfen (letzes Zeichen lschen reicht aus!)
                  }
      } 
      if (awd.stick_speed) _delay_ms(1); // USB-Speed/Timing bremsen bei Bedarf
      if (antw >= 2) {
        if(vnc_wr_byte(0x0D)) return 0;  // CR um Fehler zu besttigen, dann kommt wieder ein Prompt, bei Fehler beenden
      }
      if (antw) break;
      rd_answer_versuche = TIMEOUT_KOMMUNIKATION;   // Timeout rcksetzen
    } else rd_answer_versuche--;
  } // While
  if (!antw) antw = 99;                             // Timeout
  return antw;
}

// ***********************************************************************
// VNC1L - Auf PROMPT warten
// Bei Fehler Fehler-Nr. zurckgeben
// Rckgabe: 0  = Prompt empfangen
//           xx = Fehler-Nr. der rd_answer Funktion
// ***********************************************************************
unsigned char vnc_prompt_check(void) {
unsigned char status_1, status_2;
  status_1 = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (status_1 == 1) return 0;  // Prompt ohne vorherigen Fehler empfangen, Kmdo. wurde ausgefhrt
  // Fehler aufgetreten, nun nur auf Prompt warten, aber Fehler rckmelden
  status_2 = vnc_rd_answer(PROMPT_CHECK);
  if (status_2 == 1) return status_1;
  return status_2;              // kein Prompt, Timeout
}

// ***********************************************************************
// Fehlermeldung bewerten
// ***********************************************************************
// 1  = Kommando wurde vom VNC1L nicht akzeptiert
// 2  = Command Failed
// 3  = Bad Command
// 4  = Disk Full
// 5  = Filename Invalid
// 6  = Read Only
// 7  = File Open
// 8  = Dir Not Empty
// 9  = No Disk
// 20 = kein VDIP1/VNC1L mit VDAP-Firmware vorhanden
// 21 = kein USB-Stick angesteckt
// 22 = Timingfehler (kein Prompt)
// 23 = Initfehler
// 24 = Schreibfehler
// 50 = Buffer zu klein zum Einlesen der Datei
// 51 = Dateilnge = 0 oder > 64kByte
// 99 = Timeout
// ***********************************************************************
void vnc_fehlermeldung(unsigned char fehler_nr, unsigned char fehler_kmdo) {

  #ifdef LCD_MONITORING
    sprintf (&lcd.buffer[0],"F-Nr.:%2d (Kmdo.:%2d)",fehler_nr, fehler_kmdo);
    lcd_monitor_string(lcd.buffer); // Fehlermeldung auf LCD anzeigen
  #endif
  
  // Fehler auswerten
  clear_lcd();
  printf_lcd(0,0,"USB-Stick Fehler:",1);
  switch (fehler_nr) {
    case 20: // VDIP1 nicht vorhanden
             printf_lcd(0,1,"> Host VDIP1",1);
             printf_lcd(0,2,"  nicht vorhanden!",1);
             break;
    case 21: // kein USB-Stick angesteckt
             printf_lcd(0,1,"> kein USB-Stick",1);
             printf_lcd(0,2,"  angesteckt!",1);
             break;
    case 22: // Timingfehler
             printf_lcd(0,1,"> Timing/Timeout",1);
             printf_lcd(0,2,"  aufgetreten!",1);
             break;
    case 23: // Initfehler
             printf_lcd(0,1,"> keine Initiali-",1);
             printf_lcd(0,2,"  sierung mgl.!",1);
             break;
    case 24: // Schreibfehler
             printf_lcd(0,1,"> kein schreiben",1);
             printf_lcd(0,2,"  m\x7Cglich!",1);
             break;
    default: // Standard
             sprintf (&lcd.buffer[0],"F-Nr.:%2d (Kmdo.:%2d)",fehler_nr,fehler_kmdo);
             printf_lcd(0,1,lcd.buffer,1);
  }
  printf_lcd(0,3,"                   \x1C",1);
  get_taste(TASTEN_NOREPEAT);       // auf Tastendruck warten
}
